"use client";
import React from "react";
import {Label} from "@/components/ui/label";
import {Input} from "@/components/ui/input";
import {cn} from "@/lib/utils";
import {OctagonX, User} from "lucide-react";
import Link from "next/link";
import {Button} from "@/components/ui/button";
import {Resolver, useForm} from "react-hook-form";
import toast, {ToastBar} from "react-hot-toast";
import {ToastBox, toastType} from "@/components/ui/toast-box";

interface UserInsert {
	username: string,
	displayName?: string,
	email: string,
	password: string,
	rePassword: string
	error?: Record<string, { type: string; message: string }>
}

export const resolver: Resolver<UserInsert> = async (values) => {
	const errors: Record<string, { type: string; message: string }> = {}

	if (!values.username) {
		errors.name = {
			type: 'required',
			message: 'The username is required.',
		}
	}

	if (!values.displayName) {
		errors.description = {
			type: 'required',
			message: 'The displayname is required.',
		}
	}

	if (!values.email) {
		errors.start = {
			type: 'required',
			message: 'The email is required.',
		}
	}

	if (!values.password) {
		errors.promoId = {
			type: 'required',
			message: 'The password is required.',
		}
	}

	if (!values.rePassword) {
		errors.promoId = {
			type: 'required',
			message: 'You should retype the password.',
		}
	}

	return {
		values: Object.keys(errors).length === 0 ? values : {},
		errors,
	}
}

export function RegisterForm() {
	const handleSubbmit = (e: React.FormEvent<HTMLFormElement>) => {
		e.preventDefault();
		console.log("Form submitted");
	};

	const { register, handleSubmit, getValues } = useForm();

	const onSubmit = (data: UserInsert) => {
		console.log(data);
		if (!data.error) {
			toast.custom(<ToastBox
				message={'Hello world !'}
				type={toastType.success}
			/>)
		}
	};
	return (
		<div className="max-w-md w-full mx-auto rounded-none md:rounded-2xl p-4 md:p-8 shadow-input md:scale-125">
			<h2 className="font-bold text-xl text-neutral-800 dark:text-neutral-200">
				Create an account
			</h2>
			<p className="text-neutral-600 text-sm max-w-sm mt-2 dark:text-neutral-300">
				By creating an account you can discuss on our platform.
			</p>

			<form className="my-8" onSubmit={handleSubmit(onSubmit)}>
				<div className="flex flex-col md:flex-row space-y-2 md:space-y-0 md:space-x-2 mb-4">
					<LabelInputContainer className="mb-4">
						<Label htmlFor="username">Username<em className={"opacity-50"}>*</em></Label>
						<Input
							id="username"
							placeholder="john.doe68"
							type="text"
							{...register("username", { required: true })}
						/>
					</LabelInputContainer>
					<LabelInputContainer className="mb-4">
						<Label htmlFor="displayName">Displayname</Label>
						<Input
							id="displayName"
							placeholder="Johny"
							type="text"
							{...register("displayName")}
						/>
					</LabelInputContainer>
				</div>
				<div
					className="bg-gradient-to-r from-transparent via-neutral-300 dark:via-neutral-700 to-transparent my-4 h-[1px] w-full"/>
				<LabelInputContainer className="mb-4">
					<Label htmlFor="email">Email Address<em className={"opacity-50"}>*</em></Label>
					<Input
						id="email"
						placeholder="john.doe@foo.bar"
						type="email"
						{...register("email", { required: true })}
					/>
				</LabelInputContainer>
				<LabelInputContainer className="mb-4">
					<Label htmlFor="password">Password<em className={"opacity-50"}>*</em></Label>
					<Input
						id="password"
						placeholder="••••••••"
						type="password"
						{...register("password", { required: true })}
					/>
				</LabelInputContainer>
				<LabelInputContainer className="mb-8">
					<Label htmlFor="re-password">Re-type password<em className={"opacity-50"}>*</em></Label>
					<Input
						id="re-password"
						placeholder="••••••••"
						type="password"
						{...register("re-password", { validate: (value) => value === getValues('password') || "The passwords do not match" })}
					/>
				</LabelInputContainer>

				<button
					className="bg-gradient-to-br relative group/btn from-black dark:from-zinc-900 dark:to-zinc-900 to-neutral-600 block dark:bg-zinc-800 w-full text-white rounded-md h-10 font-medium shadow-[0px_1px_0px_0px_#ffffff40_inset,0px_-1px_0px_0px_#ffffff40_inset] dark:shadow-[0px_1px_0px_0px_var(--zinc-800)_inset,0px_-1px_0px_0px_var(--zinc-800)_inset]"
					type="submit"
				>
					Register &rarr;
					<BottomGradient/>
				</button>
				<div
					className="bg-gradient-to-r from-transparent via-neutral-300 dark:via-neutral-700 to-transparent my-8 h-[1px] w-full"/>

				<div className="flex flex-col space-y-4">
					<Button
						className=" relative group/btn flex space-x-2 items-center justify-center px-4 w-full text-black rounded-md h-10 font-medium shadow-input bg-gray-50 dark:bg-zinc-900 dark:shadow-[0px_0px_1px_1px_var(--neutral-800)]"
						asChild
					>
						<Link
							className={'flex flex-row justify-center items-center gap-2'}
							href={"/account/login"}>
							<User className="h-4 w-4 text-neutral-800 dark:text-neutral-300"/>
							<span className="text-neutral-700 dark:text-neutral-300 text-sm">
              I already have an account
            	</span>
							<BottomGradient/>
						</Link>
					</Button>
				</div>
			</form>
		</div>
	);
}

const BottomGradient = () => {
	return (
		<>
			<span className="group-hover/btn:opacity-100 block transition duration-500 opacity-0 absolute h-px w-full -bottom-px inset-x-0 bg-gradient-to-r from-transparent via-accent-foreground to-transparent" />
			<span className="group-hover/btn:opacity-100 blur-sm block transition duration-500 opacity-0 absolute h-px w-1/2 mx-auto -bottom-px inset-x-10 bg-gradient-to-r from-transparent via-primary-foreground to-transparent" />
		</>
	);
};

const LabelInputContainer = ({
															 children,
															 className,
														 }: {
	children: React.ReactNode;
	className?: string;
}) => {
	return (
		<div className={cn("flex flex-col space-y-2 w-full", className)}>
			{children}
		</div>
	);
};
